function ergebnis = climb(Func, x_0, h_min, h)
%Programm entstanden aus der Vorlesung:
%   "Fuzzy-Logik, Neuronale Netze und Evolutionaere Algorithmen"
%   bei Prof. Dr.-Ing. Jrgen Adamy
%   5. Januar 2005,
%   von Alexander Steiger
%   e-mail: steigeralex@gmx.de
%
% CLIMB Hill-Climbing-Verfahren von Hooke und Jeeves(1961)
%   climb(Func,x_0) berechnet ein Minimum einer Funktion Func.
%   Func ist ein String, das den Namen der MATLAB-function enthlt.
%   Die Funktion wird normalerweise in einem M-file definiert.
%   x_0 ist der Startpunkt.
%
%   Weitere Versionen von climb(... ):
%   climb(Func, x_0, h_min)
%   climb(Func, x_0, h_min, h)
%   
%   h_min ist annherend der Bestimmungsfehler des gefundenen Minimums.
%   h ist die Start-Schrittweite.
%
%   z.B. climb('cos',0.1])
%        climb('parabol',[0 0])
%        climb('parabol',[0 0], 0.1, 1e-10)

if nargin < 2, disp('Fehler: siehe "help climb"'); return; end

n = length(x_0);

if nargin == 2, h_min = 1e-20; h = ones(n)*0.1; end
if nargin == 3, h = ones(n)*0.1; end

x = zeros(n);
v = zeros(n);
e = eye(n);

%START *****************************************************************************

% Init
j = 1;
k = 1; %statt 0 eine 1! (Vektoren beginnen immer bei 1!)
b = 0; %false
x_d = x_0;
x(k,:) = x_0;

%Anfang Schrittweitenhalbierung
while 1==1
    %Anfang Ruecksetzen
    while 1==1
        %Anfang Extrapolation
        while 1==1
            % Anfang Tastschrittschleife
            while 1==1
                x_o = x_d + h(j)*e(j,:);
                if feval(Func,x_d) > feval(Func,x_o)
                    x_d = x_o;
                else
                    x_o = x_d - h(j)*e(j,:);
                    if feval(Func,x_d) > feval(Func,x_o)
                        x_d = x_o;
                    end    
                end    
                if j >= n       %break aus Tastschrittschleife
                    break;
                end    
                j = j + 1;
            end
            % Ende Tastschrittschleife
            if feval(Func,x_d) >= feval(Func,x(k,:))       %break aus Extrapolation
                break;
            end    
            k = k + 1;
            x(k,:) = x_d;
            v(k,:) = x(k,:) - x(k-1,:);
            x_d = x(k,:) + v(k,:);
            j = 1;
            b = 1; % true
        end
        %Ende Extrapolation
        if b == 0 %false       break aus Ruecksetzen
            break;
        end
        x_d = x(k,:);
        b = 0; %false
        j = 1;
    end
    %Ende Ruecksetzen
    if h(j) <= h_min    %break aus Schrittweitenhalbierung
        break;
    end
    h = 0.5 * h;
    j = 1;
end    
%Ende Schrittweitenhalbierung
    

%ENDE ******************************************************************************
%disp(x_d);
ergebnis = x_d;